@@ -68,6 +68,7 @@ import numpy as np
 
 from astropy import __version__, log
 from astropy.io import fits
+from astropy.io.fits import CompImageHDU
 
 DESCRIPTION = """
 Print the header(s) of a FITS file. Optional arguments allow the desired
@@ -84,8 +85,6 @@ for further documentation.
 class ExtensionNotFoundException(Exception):
     """Raised if an HDU extension requested by the user does not exist."""
 
-    pass
-
 
 class HeaderFormatter:
     """Class to format the header(s) of a FITS file for display by the
@@ -202,10 +201,10 @@ class HeaderFormatter:
         """
         # First we obtain the desired header
         try:
-            if compressed:
+            if compressed and isinstance(self._hdulist[hdukey], CompImageHDU):
                 # In the case of a compressed image, return the header before
                 # decompression (not the default behavior)
-                header = self._hdulist[hdukey]._header
+                header = self._hdulist[hdukey]._bintable.header
             else:
                 header = self._hdulist[hdukey].header
         except (IndexError, KeyError):
