@@ -2,7 +2,7 @@
 
 import os
 
-import numpy
+from numpy import get_include as get_numpy_include
 from setuptools import Extension
 
 BLS_ROOT = os.path.relpath(os.path.dirname(__file__))
@@ -11,11 +11,10 @@ BLS_ROOT = os.path.relpath(os.path.dirname(__file__))
 def get_extensions():
     ext = Extension(
         "astropy.timeseries.periodograms.bls._impl",
-        define_macros=[("NPY_NO_DEPRECATED_API", "NPY_1_7_API_VERSION")],
         sources=[
             os.path.join(BLS_ROOT, "bls.c"),
             os.path.join(BLS_ROOT, "_impl.pyx"),
         ],
-        include_dirs=[numpy.get_include()],
+        include_dirs=[get_numpy_include()],
     )
     return [ext]
