@@ -3,20 +3,38 @@
 
 import abc
 import numbers
+from collections.abc import Sequence
 from itertools import zip_longest
+from math import prod
+from types import EllipsisType
+from typing import Self, TypeVar
 
 import numpy as np
+from numpy.typing import NDArray
+
+from astropy.utils.compat import NUMPY_LT_2_0
+from astropy.utils.decorators import deprecated
+
+if NUMPY_LT_2_0:
+    import numpy.core as np_core
+    from numpy.core.multiarray import normalize_axis_index
+else:
+    import numpy._core as np_core
+    from numpy.lib.array_utils import normalize_axis_index
 
 __all__ = [
+    "IncompatibleShapeError",
     "NDArrayShapeMethods",
     "ShapedLikeNDArray",
     "check_broadcast",
-    "IncompatibleShapeError",
     "simplify_basic_index",
     "unbroadcast",
 ]
 
 
+DT = TypeVar("DT", bound=np.generic)
+
+
 class NDArrayShapeMethods:
     """Mixin class to provide shape-changing methods.
 
@@ -93,7 +111,7 @@ class NDArrayShapeMethods:
         return self._apply("transpose", *args, **kwargs)
 
     @property
-    def T(self):
+    def T(self) -> Self:
         """Return an instance with the data transposed.
 
         Parameters are as for :attr:`~numpy.ndarray.T`.  All internal
@@ -170,7 +188,7 @@ class ShapedLikeNDArray(NDArrayShapeMethods, metaclass=abc.ABCMeta):
 
     @property
     @abc.abstractmethod
-    def shape(self):
+    def shape(self) -> tuple[int, ...]:
         """The shape of the underlying data."""
 
     @abc.abstractmethod
@@ -195,28 +213,25 @@ class ShapedLikeNDArray(NDArrayShapeMethods, metaclass=abc.ABCMeta):
         """
 
     @property
-    def ndim(self):
+    def ndim(self) -> int:
         """The number of dimensions of the instance and underlying arrays."""
         return len(self.shape)
 
     @property
-    def size(self):
+    def size(self) -> int:
         """The size of the object, as calculated from its shape."""
-        size = 1
-        for sh in self.shape:
-            size *= sh
-        return size
+        return prod(self.shape)
 
     @property
-    def isscalar(self):
+    def isscalar(self) -> bool:
         return self.shape == ()
 
-    def __len__(self):
+    def __len__(self) -> int:
         if self.isscalar:
             raise TypeError(f"Scalar {self.__class__.__name__!r} object has no len()")
         return self.shape[0]
 
-    def __bool__(self):
+    def __bool__(self) -> bool:
         """Any instance should evaluate to True, except when it is empty."""
         return self.size > 0
 
@@ -275,7 +290,7 @@ class ShapedLikeNDArray(NDArrayShapeMethods, metaclass=abc.ABCMeta):
             "alltrue": "all",
             "sometrue": "any",
         }.get(name, name)
-        for name in np.core.fromnumeric.__all__
+        for name in np_core.fromnumeric.__all__
         if name not in ["alen", "sort", "partition"]
     }
     # Add np.copy, which we may as well let defer to our method.
@@ -324,11 +339,18 @@ class ShapedLikeNDArray(NDArrayShapeMethods, metaclass=abc.ABCMeta):
 
 
 class IncompatibleShapeError(ValueError):
-    def __init__(self, shape_a, shape_a_idx, shape_b, shape_b_idx):
+    def __init__(
+        self,
+        shape_a: tuple[int, ...],
+        shape_a_idx: int,
+        shape_b: tuple[int, ...],
+        shape_b_idx: int,
+    ) -> None:
         super().__init__(shape_a, shape_a_idx, shape_b, shape_b_idx)
 
 
-def check_broadcast(*shapes):
+@deprecated("7.0", alternative="np.broadcast_shapes")
+def check_broadcast(*shapes: tuple[int, ...]) -> tuple[int, ...]:
     """
     Determines whether two or more Numpy arrays can be broadcast with each
     other based on their shape tuple alone.
@@ -376,7 +398,7 @@ def check_broadcast(*shapes):
     return tuple(full_shape[::-1])
 
 
-def unbroadcast(array):
+def unbroadcast(array: NDArray[DT]) -> NDArray[DT]:
     """
     Given an array, return a new array that is the smallest subset of the
     original array that can be re-broadcasted back to the original array.
@@ -399,7 +421,11 @@ def unbroadcast(array):
     return array.reshape(array.shape[first_not_unity:])
 
 
-def simplify_basic_index(basic_index, *, shape):
+def simplify_basic_index(
+    basic_index: int | slice | Sequence[int | slice | EllipsisType | None],
+    *,
+    shape: Sequence[int],
+) -> tuple[int | slice, ...]:
     """
     Given a Numpy basic index, return a tuple of integers and slice objects
     with no default values (`None`) if possible.
@@ -455,9 +481,7 @@ def simplify_basic_index(basic_index, *, shape):
                     indices[1] = None
                 new_index[i] = slice(*indices)
             elif isinstance(slc, numbers.Integral):
-                new_index[i] = np.core.multiarray.normalize_axis_index(
-                    int(slc), shape[i]
-                )
+                new_index[i] = normalize_axis_index(int(slc), shape[i])
             else:
                 raise ValueError(f"Unexpected index element in basic index: {slc}")
         else:
