@@ -1,16 +1,16 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """
 This module provides the tools used to internally run the astropy test suite
-from the installed astropy.  It makes use of the `pytest`_ testing framework.
+from the installed astropy.  It makes use of the |pytest| testing framework.
 """
+
 import os
 import pickle
+import sys
 
 import pytest
 
 from astropy.units import allclose as quantity_allclose  # noqa: F401
-from astropy.utils.compat import PYTHON_LT_3_11
-from astropy.utils.introspection import minversion
 
 # For backward-compatibility with affiliated packages
 from .runner import TestRunner  # noqa: F401
@@ -19,11 +19,10 @@ __all__ = [
     "assert_follows_unicode_guidelines",
     "assert_quantity_allclose",
     "check_pickling_recovery",
-    "pickle_protocol",
     "generic_recursive_equality_test",
+    "pickle_protocol",
 ]
 
-PYTEST_LT_8_0 = not minversion(pytest, "8.0")
 
 # https://docs.github.com/en/actions/learn-github-actions/variables#default-environment-variables
 CI = os.environ.get("CI", "false") == "true"
@@ -127,10 +126,17 @@ def assert_follows_unicode_guidelines(x, roundtrip=None):
             assert eval(repr_x, roundtrip) == x
 
 
-@pytest.fixture(params=[0, 1, -1])
+@pytest.fixture(params=[0, 1, 4, -1])
 def pickle_protocol(request):
     """
-    Fixture to run all the tests for protocols 0 and 1, and -1 (most advanced).
+    Fixture to run all the tests for protocols 0, 1, 4 and -1 (most advanced).
+
+    * 0 is the original "human-readable" protocol
+    * 1 is the oldest binary format
+    * 4 introduced in Python 3.4, has been the default in 3.8-3.13
+    * -1 is the most advanced, which is 5 since Python 3.8, default from 3.14,
+      and the first binary format to preserve byteorder.
+
     (Originally from astropy.table.tests.test_pickle).
     """
     return request.param
@@ -141,15 +147,12 @@ def generic_recursive_equality_test(a, b, class_history):
     Check if the attributes of a and b are equal. Then,
     check if the attributes of the attributes are equal.
     """
-    if PYTHON_LT_3_11:
-        dict_a = a.__getstate__() if hasattr(a, "__getstate__") else a.__dict__
-    else:
-        # NOTE: The call may need to be adapted if other objects implementing a __getstate__
-        # with required argument(s) are passed to this function.
-        # For a class with `__slots__` the default state is not a `dict`;
-        # with neither `__dict__` nor `__slots__` it is `None`.
-        state = a.__getstate__(a) if isinstance(a, type) else a.__getstate__()
-        dict_a = state if isinstance(state, dict) else getattr(a, "__dict__", dict())
+    # NOTE: The call may need to be adapted if other objects implementing a __getstate__
+    # with required argument(s) are passed to this function.
+    # For a class with `__slots__` the default state is not a `dict`;
+    # with neither `__dict__` nor `__slots__` it is `None`.
+    state = a.__getstate__(a) if isinstance(a, type) else a.__getstate__()
+    dict_a = state if isinstance(state, dict) else getattr(a, "__dict__", {})
     dict_b = b.__dict__
     for key in dict_a:
         assert key in dict_b, f"Did not pickle {key}"
@@ -199,6 +202,12 @@ def assert_quantity_allclose(actual, desired, rtol=1.0e-7, atol=None, **kwargs):
 
     from astropy.units.quantity import _unquantify_allclose_arguments
 
+    __tracebackhide__ = True
     np.testing.assert_allclose(
         *_unquantify_allclose_arguments(actual, desired, rtol, atol), **kwargs
     )
+
+
+_skip_docstring_tests_with_optimized_python = pytest.mark.skipif(
+    sys.flags.optimize >= 2, reason="docstrings are not testable in optimized mode"
+)
