@@ -11,14 +11,14 @@ from fractions import Fraction
 
 import numpy as np
 
-from astropy.units.core import (
-    UnitConversionError,
-    UnitsError,
-    UnitTypeError,
-    dimensionless_unscaled,
-    unit_scale_converter,
+from astropy.units.core import dimensionless_unscaled, unit_scale_converter
+from astropy.units.errors import UnitConversionError, UnitsError, UnitTypeError
+from astropy.utils.compat.numpycompat import (
+    NUMPY_LT_2_0,
+    NUMPY_LT_2_1,
+    NUMPY_LT_2_2,
+    NUMPY_LT_2_3,
 )
-from astropy.utils.compat.numpycompat import NUMPY_LT_2_0
 
 if NUMPY_LT_2_0:
     from numpy.core import umath as np_umath
@@ -36,10 +36,27 @@ def _d(unit):
 
 
 def get_converter(from_unit, to_unit):
-    """Like Unit._get_converter, except returns None if no scaling is needed,
+    """Like Unit.get_converter, except returns None if no scaling is needed,
     i.e., if the inferred scale is unity.
     """
-    converter = from_unit._get_converter(to_unit)
+    try:
+        converter = from_unit.get_converter(to_unit)
+    except AttributeError as exc:
+        # Check for lack of unit only now, to avoid delay for cases where a unit
+        # was present. Note that cases where dimensionless is expected are
+        # already short-circuited; here, we cover just the case where, e.g., the
+        # user has done u.add_enabled_equivalencies(u.dimensionless_angles()).
+        if from_unit is not None:  # pragma: no cover
+            raise
+        try:
+            converter = dimensionless_unscaled.get_converter(to_unit)
+        except UnitsError:
+            exc.add_note(
+                "Input without a 'unit' attribute? Such input is treated "
+                f"as dimensionless and cannot be converted to {to_unit}."
+            )
+            raise exc
+
     return None if converter is unit_scale_converter else converter
 
 
@@ -390,10 +407,21 @@ if not NUMPY_LT_2_0:
         np._core.umath._replace,
         np._core.umath._expandtabs,
         np._core.umath._expandtabs_length,
+    }
+if not NUMPY_LT_2_1:
+    UNSUPPORTED_UFUNCS |= {
         np._core.umath._ljust,
         np._core.umath._rjust,
         np._core.umath._center,
         np._core.umath._zfill,
+        np._core.umath._partition_index,
+        np._core.umath._rpartition,
+        np._core.umath._rpartition_index,
+        np._core.umath._partition,
+    }
+if not NUMPY_LT_2_3:
+    UNSUPPORTED_UFUNCS |= {
+        np._core.umath._slice,
     }
 
 # SINGLE ARGUMENT UFUNCS
@@ -525,8 +553,11 @@ for ufunc in twoarg_invtrig_ufuncs:
 # ufuncs handled as special cases
 UFUNC_HELPERS[np.multiply] = helper_multiplication
 UFUNC_HELPERS[np.matmul] = helper_multiplication
-if isinstance(getattr(np, "vecdot", None), np.ufunc):
+if not NUMPY_LT_2_0:
     UFUNC_HELPERS[np.vecdot] = helper_multiplication
+if not NUMPY_LT_2_2:
+    UFUNC_HELPERS[np.vecmat] = helper_multiplication
+    UFUNC_HELPERS[np.matvec] = helper_multiplication
 UFUNC_HELPERS[np.divide] = helper_division
 UFUNC_HELPERS[np.true_divide] = helper_division
 UFUNC_HELPERS[np.power] = helper_power
