@@ -1,24 +1,17 @@
 import difflib
 import functools
-import numbers
 import sys
+from textwrap import indent
 
 import numpy as np
 
-from .misc import indent
-
 __all__ = [
-    "fixed_width_indent",
     "diff_values",
     "report_diff_values",
     "where_not_allclose",
 ]
 
 
-# Smaller default shift-width for indent
-fixed_width_indent = functools.partial(indent, width=2)
-
-
 def diff_values(a, b, rtol=0.0, atol=0.0):
     """
     Diff two scalar values. If both values are floats, they are compared to
@@ -86,11 +79,10 @@ def report_diff_values(a, b, fileobj=sys.stdout, indent_width=0, rtol=0.0, atol=
         `True` if no diff, else `False`.
 
     """
+    indent_prefix = indent_width * "  "
     if isinstance(a, np.ndarray) and isinstance(b, np.ndarray):
         if a.shape != b.shape:
-            fileobj.write(
-                fixed_width_indent("  Different array shapes:\n", indent_width)
-            )
+            fileobj.write(indent("  Different array shapes:\n", indent_prefix))
             report_diff_values(
                 str(a.shape),
                 str(b.shape),
@@ -108,7 +100,7 @@ def report_diff_values(a, b, fileobj=sys.stdout, indent_width=0, rtol=0.0, atol=
 
         for idx in diff_indices[:3]:
             lidx = idx.tolist()
-            fileobj.write(fixed_width_indent(f"  at {lidx!r}:\n", indent_width))
+            fileobj.write(indent(f"  at {lidx!r}:\n", indent_prefix))
             report_diff_values(
                 a[tuple(idx)],
                 b[tuple(idx)],
@@ -120,9 +112,7 @@ def report_diff_values(a, b, fileobj=sys.stdout, indent_width=0, rtol=0.0, atol=
 
         if num_diffs > 3:
             fileobj.write(
-                fixed_width_indent(
-                    f"  ...and at {num_diffs - 3:d} more indices.\n", indent_width
-                )
+                indent(f"  ...and at {num_diffs - 3:d} more indices.\n", indent_prefix)
             )
             return False
 
@@ -135,12 +125,8 @@ def report_diff_values(a, b, fileobj=sys.stdout, indent_width=0, rtol=0.0, atol=
         lnpad = " "
         sign_a = "a>"
         sign_b = "b>"
-        if isinstance(a, numbers.Number):
-            a = repr(a)
-            b = repr(b)
-        else:
-            a = str(a)
-            b = str(b)
+        a = str(a)
+        b = str(b)
     else:
         padding = max(len(typea.__name__), len(typeb.__name__)) + 3
         lnpad = (padding + 1) * " "
@@ -149,22 +135,8 @@ def report_diff_values(a, b, fileobj=sys.stdout, indent_width=0, rtol=0.0, atol=
 
         is_a_str = isinstance(a, str)
         is_b_str = isinstance(b, str)
-        a = (
-            repr(a)
-            if (
-                (is_a_str and not is_b_str)
-                or (not is_a_str and isinstance(a, numbers.Number))
-            )
-            else str(a)
-        )
-        b = (
-            repr(b)
-            if (
-                (is_b_str and not is_a_str)
-                or (not is_b_str and isinstance(b, numbers.Number))
-            )
-            else str(b)
-        )
+        a = repr(a) if is_a_str and not is_b_str else str(a)
+        b = repr(b) if is_b_str and not is_a_str else str(b)
 
     identical = True
 
@@ -177,14 +149,12 @@ def report_diff_values(a, b, fileobj=sys.stdout, indent_width=0, rtol=0.0, atol=
             line = sign_b + line[1:]
         else:
             line = lnpad + line
-        fileobj.write(
-            fixed_width_indent("  {}\n".format(line.rstrip("\n")), indent_width)
-        )
+        fileobj.write(indent("  {}\n".format(line.rstrip("\n")), indent_prefix))
 
     return identical
 
 
-def where_not_allclose(a, b, rtol=1e-5, atol=1e-8):
+def where_not_allclose(a, b, rtol=1e-5, atol=1e-8, return_maxdiff=False):
     """
     A version of :func:`numpy.allclose` that returns the indices
     where the two arrays differ, instead of just a boolean value.
@@ -193,25 +163,50 @@ def where_not_allclose(a, b, rtol=1e-5, atol=1e-8):
     ----------
     a, b : array-like
         Input arrays to compare.
-
     rtol, atol : float
         Relative and absolute tolerances as accepted by
         :func:`numpy.allclose`.
+    return_maxdiff : bool
+        Return the maximum of absolute and relative differences.
 
     Returns
     -------
     idx : tuple of array
         Indices where the two arrays differ.
+    max_absolute : float
+        Maximum of absolute difference, returned if ``return_maxdiff=True``.
+    max_relative : float
+        Maximum of relative difference, returned if ``return_maxdiff=True``.
 
     """
     # Create fixed mask arrays to handle INF and NaN; currently INF and NaN
     # are handled as equivalent
-    if not np.all(np.isfinite(a)):
-        a = np.ma.fix_invalid(a).data
-    if not np.all(np.isfinite(b)):
-        b = np.ma.fix_invalid(b).data
+    a = np.ma.masked_invalid(a)
+    b = np.ma.masked_invalid(b)
+
+    absolute = np.ma.abs(b - a)
 
     if atol == 0.0 and rtol == 0.0:
         # Use a faster comparison for the most simple (and common) case
-        return np.where(a != b)
-    return np.where(np.abs(a - b) > (atol + rtol * np.abs(b)))
+        thresh = 0
+    else:
+        thresh = atol + rtol * np.abs(b)
+
+    # values invalid in only one of the two arrays should be reported
+    invalid = a.mask ^ b.mask
+    indices = np.where(invalid | (absolute.filled(0) > thresh))
+
+    if return_maxdiff:
+        absolute[invalid] = np.ma.masked
+        finites = ~absolute.mask
+        absolute = absolute.compressed()
+        if len(indices[0]) == 0 or absolute.size == 0:
+            max_absolute = max_relative = 0
+        else:
+            # remove all invalid values before computing max differences
+            relative = absolute / np.abs(b[finites])
+            max_absolute = float(np.max(absolute))
+            max_relative = np.max(relative)
+        return indices, max_absolute, max_relative
+    else:
+        return indices
