@@ -292,6 +292,36 @@ def test_slice_wcs():
         mywcs[0, ::2]
 
 
+def test_slice_nd():
+    # Regression test for a bug that caused slicing to not work correctly for
+    # WCS with more than 2 dimensions.
+
+    sub = WCS(naxis=3)[:, :, :]
+    assert isinstance(sub, WCS)
+
+
+def test_slice_ellipsis():
+    # Regression test for a bug that caused slicing with an ellipsis to not
+    # return a WCS object but a SlicedFITSWCS instead
+
+    sub = WCS(naxis=3)[...]
+    assert isinstance(sub, WCS)
+
+
+def test_slice_drop_dimensions_order():
+    # Regression test for a bug that caused WCS.slice to ignore
+    # ``numpy_order=False`` if dimensions were dropped.
+
+    wcs = WCS(naxis=3)
+    wcs.wcs.ctype = "RA---TAN", "DEC--TAN", "FREQ"
+
+    wcs_sliced_1 = wcs.slice([0, slice(None), slice(None)], numpy_order=True)
+    assert wcs_sliced_1.world_axis_physical_types == ["pos.eq.ra", "pos.eq.dec"]
+
+    wcs_sliced_2 = wcs.slice([slice(None), slice(None), 0], numpy_order=False)
+    assert wcs_sliced_2.world_axis_physical_types == ["pos.eq.ra", "pos.eq.dec"]
+
+
 def test_axis_names():
     mywcs = WCS(naxis=4)
     mywcs.wcs.ctype = ["RA---TAN", "DEC--TAN", "VOPT-LSR", "STOKES"]
@@ -932,6 +962,38 @@ def test_local_pixel_derivatives(spatial_wcs_2d_small_angle):
     np.testing.assert_allclose(derivs[not_diag].flat, [0, 0], atol=1e-8)
 
 
+def test_local_pixel_derivatives_cube():
+    cube_wcs = WCS(naxis=3)
+    cube_wcs.wcs.ctype = "RA---TAN", "DEC--TAN", "FREQ"
+    cube_wcs.wcs.crval = 10, 20, 30
+    cube_wcs.wcs.cdelt = 0.0001, 0.0001, 0.01
+    cube_wcs.wcs.cunit = "deg", "deg", "GHz"
+    cube_wcs.wcs.set()
+
+    derivs = local_partial_pixel_derivatives(cube_wcs, 0, 0, 0)
+    np.testing.assert_allclose(
+        derivs, [[0.0001, 0, 0], [0, 0.0001, 0], [0, 0, 1e7]], rtol=0.1, atol=1e-5
+    )
+
+    derivs = local_partial_pixel_derivatives(cube_wcs, 0, 0, 0, normalize_by_world=True)
+    np.testing.assert_allclose(
+        derivs, [[1, 0, 0], [0, 1, 0], [0, 0, 1]], rtol=0.1, atol=1e-5
+    )
+
+    # Slice WCS so that there are two pixel and three world coordinates
+    sliced_wcs = cube_wcs[:, 0, :]
+
+    derivs = local_partial_pixel_derivatives(sliced_wcs, 0, 0, 0)
+    np.testing.assert_allclose(
+        derivs, [[0.0001, 0], [0, 0], [0, 1e7]], rtol=0.1, atol=1e-5
+    )
+
+    derivs = local_partial_pixel_derivatives(
+        sliced_wcs, 0, 0, 0, normalize_by_world=True
+    )
+    np.testing.assert_allclose(derivs, [[1, 0], [1, 0], [0, 1]], rtol=0.1, atol=1e-5)
+
+
 def test_pixel_to_world_correlation_matrix_celestial():
     wcs = WCS(naxis=2)
     wcs.wcs.ctype = "RA---TAN", "DEC--TAN"
