@@ -4,6 +4,7 @@ import io
 import os
 from contextlib import nullcontext
 from datetime import datetime
+from multiprocessing.pool import ThreadPool
 
 import numpy as np
 import pytest
@@ -21,7 +22,7 @@ from astropy.coordinates import SkyCoord
 from astropy.io import fits
 from astropy.io.fits.verify import VerifyWarning
 from astropy.nddata import Cutout2D
-from astropy.tests.helper import PYTEST_LT_8_0, assert_quantity_allclose
+from astropy.tests.helper import assert_quantity_allclose
 from astropy.utils.data import (
     get_pkg_data_contents,
     get_pkg_data_filename,
@@ -137,9 +138,10 @@ def test_fixes():
     """
     header = get_pkg_data_contents("data/nonstandard_units.hdr", encoding="binary")
 
-    with pytest.raises(wcs.InvalidTransformError), pytest.warns(
-        wcs.FITSFixedWarning
-    ) as w:
+    with (
+        pytest.raises(wcs.InvalidTransformError),
+        pytest.warns(wcs.FITSFixedWarning) as w,
+    ):
         wcs.WCS(header, translate_units="dhs")
 
     if Version("7.4") <= _WCSLIB_VER < Version("7.6"):
@@ -969,14 +971,12 @@ def test_sip_tpv_agreement():
         os.path.join("data", "tpvonly.hdr"), encoding="binary"
     )
 
-    if PYTEST_LT_8_0:
-        ctx = nullcontext()
-    else:
-        ctx = pytest.warns(
+    with (
+        pytest.warns(wcs.FITSFixedWarning),
+        pytest.warns(
             AstropyWarning, match="Some non-standard WCS keywords were excluded"
-        )
-
-    with pytest.warns(wcs.FITSFixedWarning), ctx:
+        ),
+    ):
         w_sip = wcs.WCS(sip_header)
         w_tpv = wcs.WCS(tpv_header)
 
@@ -1013,21 +1013,18 @@ def test_tpv_ctype_sip():
     sip_header["CTYPE1"] = "RA---TAN-SIP"
     sip_header["CTYPE2"] = "DEC--TAN-SIP"
 
-    if PYTEST_LT_8_0:
-        ctx1 = ctx2 = nullcontext()
-    else:
-        ctx1 = pytest.warns(
-            wcs.FITSFixedWarning, match=".*RADECSYS keyword is deprecated, use RADESYSa"
-        )
-        ctx2 = pytest.warns(
-            wcs.FITSFixedWarning, match=".*Set MJD-OBS to .* from DATE-OBS"
-        )
-
-    with pytest.warns(
-        wcs.FITSFixedWarning,
-        match="Removed redundant SCAMP distortion parameters "
-        "because SIP parameters are also present",
-    ), ctx1, ctx2:
+    with (
+        pytest.warns(
+            wcs.FITSFixedWarning,
+            match="Removed redundant SCAMP distortion parameters "
+            "because SIP parameters are also present",
+        ),
+        pytest.warns(
+            wcs.FITSFixedWarning,
+            match=".*RADECSYS keyword is deprecated, use RADESYSa",
+        ),
+        pytest.warns(wcs.FITSFixedWarning, match=".*Set MJD-OBS to .* from DATE-OBS"),
+    ):
         w_sip = wcs.WCS(sip_header)
 
     assert w_sip.sip is not None
@@ -1044,21 +1041,18 @@ def test_tpv_ctype_tpv():
     sip_header["CTYPE1"] = "RA---TPV"
     sip_header["CTYPE2"] = "DEC--TPV"
 
-    if PYTEST_LT_8_0:
-        ctx1 = ctx2 = nullcontext()
-    else:
-        ctx1 = pytest.warns(
-            wcs.FITSFixedWarning, match=".*RADECSYS keyword is deprecated, use RADESYSa"
-        )
-        ctx2 = pytest.warns(
-            wcs.FITSFixedWarning, match=".*Set MJD-OBS to .* from DATE-OBS"
-        )
-
-    with pytest.warns(
-        wcs.FITSFixedWarning,
-        match="Removed redundant SIP distortion parameters "
-        "because CTYPE explicitly specifies TPV distortions",
-    ), ctx1, ctx2:
+    with (
+        pytest.warns(
+            wcs.FITSFixedWarning,
+            match="Removed redundant SIP distortion parameters "
+            "because CTYPE explicitly specifies TPV distortions",
+        ),
+        pytest.warns(
+            wcs.FITSFixedWarning,
+            match=".*RADECSYS keyword is deprecated, use RADESYSa",
+        ),
+        pytest.warns(wcs.FITSFixedWarning, match=".*Set MJD-OBS to .* from DATE-OBS"),
+    ):
         w_sip = wcs.WCS(sip_header)
 
     assert w_sip.sip is None
@@ -1075,21 +1069,18 @@ def test_tpv_ctype_tan():
     sip_header["CTYPE1"] = "RA---TAN"
     sip_header["CTYPE2"] = "DEC--TAN"
 
-    if PYTEST_LT_8_0:
-        ctx1 = ctx2 = nullcontext()
-    else:
-        ctx1 = pytest.warns(
-            wcs.FITSFixedWarning, match=".*RADECSYS keyword is deprecated, use RADESYSa"
-        )
-        ctx2 = pytest.warns(
-            wcs.FITSFixedWarning, match=".*Set MJD-OBS to .* from DATE-OBS"
-        )
-
-    with pytest.warns(
-        wcs.FITSFixedWarning,
-        match="Removed redundant SIP distortion parameters "
-        "because SCAMP' PV distortions are also present",
-    ), ctx1, ctx2:
+    with (
+        pytest.warns(
+            wcs.FITSFixedWarning,
+            match="Removed redundant SIP distortion parameters "
+            "because SCAMP' PV distortions are also present",
+        ),
+        pytest.warns(
+            wcs.FITSFixedWarning,
+            match=".*RADECSYS keyword is deprecated, use RADESYSa",
+        ),
+        pytest.warns(wcs.FITSFixedWarning, match=".*Set MJD-OBS to .* from DATE-OBS"),
+    ):
         w_sip = wcs.WCS(sip_header)
 
     assert w_sip.sip is None
@@ -1434,6 +1425,25 @@ def test_naxis():
     assert w.pixel_bounds is None
 
 
+def test_naxis_1d():
+    w = wcs.WCS(
+        {
+            "naxis1": 100,
+            "crval1": 1,
+            "cdelt1": 0.1,
+            "crpix1": 1,
+        }
+    )
+    assert w.pixel_shape == (100,)
+    assert w.array_shape == (100,)
+
+    w.pixel_shape = (99,)
+    assert w._naxis == [99]
+
+    w.pixel_shape = None
+    assert w.pixel_bounds is None
+
+
 def test_sip_with_altkey():
     """
     Test that when creating a WCS object using a key, CTYPE with
@@ -1460,14 +1470,13 @@ def test_to_fits_1():
     """
     fits_name = get_pkg_data_filename("data/dist.fits")
 
-    if PYTEST_LT_8_0:
-        ctx = nullcontext()
-    else:
-        ctx = pytest.warns(
-            wcs.FITSFixedWarning, match="The WCS transformation has more axes"
-        )
-
-    with pytest.warns(AstropyDeprecationWarning), ctx:
+    with (
+        pytest.warns(AstropyDeprecationWarning),
+        pytest.warns(
+            wcs.FITSFixedWarning,
+            match="The WCS transformation has more axes",
+        ),
+    ):
         w = wcs.WCS(fits_name)
     wfits = w.to_fits()
     assert isinstance(wfits, fits.HDUList)
@@ -1738,17 +1747,12 @@ def test_distortion_header(tmp_path):
     """
     path = get_pkg_data_filename("data/dss.14.29.56-62.41.05.fits.gz")
     cen = np.array((50, 50))
-    siz = np.array((20, 20))
-
-    if PYTEST_LT_8_0:
-        ctx = nullcontext()
-    else:
-        ctx = pytest.warns(VerifyWarning)
+    size = np.array((20, 20))
 
     with fits.open(path) as hdulist:
-        with ctx, pytest.warns(wcs.FITSFixedWarning):
+        with pytest.warns(VerifyWarning), pytest.warns(wcs.FITSFixedWarning):
             w = wcs.WCS(hdulist[0].header)
-        cut = Cutout2D(hdulist[0].data, position=cen, size=siz, wcs=w)
+        cut = Cutout2D(hdulist[0].data, position=cen, size=size, wcs=w)
 
     # This converts the DSS plate solution model with AMD[XY]n coefficients into a
     # Template Polynomial Distortion model (TPD.FWD.n coefficients);
@@ -1773,7 +1777,7 @@ def test_distortion_header(tmp_path):
     assert w.pixel_to_world(*cen).separation(w0.pixel_to_world(*cen)) < 1.0e-3 * u.mas
 
     assert (
-        w.pixel_to_world(*cen).separation(w1.pixel_to_world(*(siz / 2)))
+        w.pixel_to_world(*cen).separation(w1.pixel_to_world(*(size / 2)))
         < 1.0e-3 * u.mas
     )
 
@@ -1784,7 +1788,7 @@ def test_distortion_header(tmp_path):
         w2 = wcs.WCS(hdulist[0].header)
 
     assert (
-        w.pixel_to_world(*cen).separation(w2.pixel_to_world(*(siz / 2)))
+        w.pixel_to_world(*cen).separation(w2.pixel_to_world(*(size / 2)))
         < 1.0e-3 * u.mas
     )
 
@@ -1857,3 +1861,116 @@ def test_swapaxes_same_val_roundtrip():
 
     # check round-tripping:
     assert np.allclose(w.wcs_world2pix([val_ref], 0)[0], imcoord, rtol=0, atol=1e-8)
+
+
+def test_DistortionLookupTable():
+    img_world_wcs = wcs.WCS(naxis=2)
+    # A simple "pixel coordinates are world coordinates" WCS, to which we'll
+    # add distortion lookup tables
+    img_world_wcs.wcs.crpix = 1, 1
+    img_world_wcs.wcs.crval = 0, 0
+    img_world_wcs.wcs.cdelt = 1, 1
+
+    # Create maps with zero distortion except at one particular pixel
+    x_dist_array = np.zeros((25, 25))
+    x_dist_array[10, 20] = 0.5
+    map_x = wcs.DistortionLookupTable(
+        x_dist_array.astype(np.float32), (5, 10), (10, 20), (2, 2)
+    )
+
+    y_dist_array = np.zeros((25, 25))
+    y_dist_array[10, 5] = 0.7
+    map_y = wcs.DistortionLookupTable(
+        y_dist_array.astype(np.float32), (5, 10), (10, 20), (3, 3)
+    )
+
+    img_world_wcs.cpdis1 = map_x
+    img_world_wcs.cpdis2 = map_y
+
+    # The x distortion of 0.5 pixels should appear at a specific spot in the
+    # image, and we need to work out what that is so we can check it. The
+    # distortion is at array index (10, 20), which is a 1-based pixel
+    # coordinate of (21, 11) and therefore at an offset of (16, 1) from the
+    # lookup table's CRPIX of (5, 10). CDELT is 2 image pixels / distortion
+    # pixel, so the distortion applies to the image pixel which is at an offset
+    # of (32, 2) from the CRVAL of (10, 20), meaning we should see the
+    # distortion at the 1-based image pixel coordinate of (42, 22).
+    assert_allclose(map_x.get_offset(42, 22), 0.5)
+    # And we should see it applied at the 0-based coordinate (41, 21) with our
+    # simple img<->world wcs.
+    assert_allclose(img_world_wcs.pixel_to_world_values(41, 21), [41.5, 21])
+    # Similarly for the y distortion, the distortion is at array index (10, 5),
+    # which is a 1-based pixel coordinate of (6, 11) and therefore at an offset
+    # of (1, 1) from the lookup table's CRPIX of (5, 10). CDELT is 3 image
+    # pixels / distortion pixel, so the distortion applies to the image pixel
+    # which is at an offset of (3, 3) from the CRVAL of (10, 20), meaning we
+    # should see the distortion at the 1-based image pixel coordinate of (13,
+    # 23).
+    assert_allclose(map_y.get_offset(13, 23), 0.7)
+    # And we should see it applied at the 0-based coordinate (12, 22) with our
+    # simple img<->world wcs.
+    assert_allclose(img_world_wcs.pixel_to_world_values(12, 22), [12, 22.7])
+
+    # Now check that when we move the image location by the equivalent of 1/2
+    # distortion-array pixel, we see only half the distortion.
+    for dx, dy in [(0.5, 0), (-0.5, 0), (0, 0.5), (0, -0.5)]:
+        # Scale dx, dy by 2 for the CDELT in the x distortion table (since
+        # we're looking for the x distortion).
+        assert_allclose(
+            img_world_wcs.pixel_to_world_values(41 + dx * 2, 21 + dy * 2),
+            [41 + dx * 2 + 0.25, 21 + dy * 2],
+        )
+        # Scale dx, dy by 3 for the CDELT in the y distortion table (since
+        # we're looking for the y distortion).
+        assert_allclose(
+            img_world_wcs.pixel_to_world_values(12 + dx * 3, 22 + dy * 3),
+            [12 + dx * 3, 22 + dy * 3 + 0.35],
+        )
+
+    # Now check that when we move the image location by the equivalent of 1
+    # distortion-array pixel, we see no distortion.
+    for dx, dy in [(2, 0), (-2, 0), (0, 2), (0, -2)]:
+        # Scale dx, dy by 2 for the CDELT in the x distortion table (since
+        # we're looking for the x distortion).
+        assert_allclose(
+            img_world_wcs.pixel_to_world_values(41 + dx * 2, 21 + dy * 2),
+            [41 + dx * 2, 21 + dy * 2],
+        )
+        # Scale dx, dy by 3 for the CDELT in the y distortion table (since
+        # we're looking for the y distortion).
+        assert_allclose(
+            img_world_wcs.pixel_to_world_values(12 + dx * 3, 22 + dy * 3),
+            [12 + dx * 3, 22 + dy * 3],
+        )
+
+
+def test_thread_safe_conversions():
+    # This is a regression test for a bug which caused wcsset to be called
+    # unnecessarily multiple times, including every time some attribute were
+    # accessed on the WCS. This meant that if one was doing a series of
+    # coordinate transforms in a multi-threaded environment, wcsset could
+    # get called in the process and modify the WCS object, which was not
+    # thread-safe. Now wcsset is not actually called after the initial time.
+    # This was discussed in more detail in https://github.com/astropy/astropy/issues/16245
+
+    w = wcs.WCS(naxis=2)
+    w.wcs.crpix = [-234.75, 8.3393]
+    w.wcs.cdelt = np.array([-0.066667, 0.066667])
+    w.wcs.crval = [0, -90]
+    w.wcs.ctype = ["RA---AIR", "DEC--AIR"]
+    w.wcs.set()
+
+    N = 1_000_000
+
+    pixel = np.random.randint(-1000, 1000, N * 2).reshape((N, 2)).astype(float)
+
+    def round_trip_transform(pixel):
+        world = w.wcs.p2s(pixel.copy(), 1)["world"]
+        w.wcs.lng  # this access causes issues, without it all works
+        pixel = w.wcs.s2p(world, 1)["pixcrd"]
+        return pixel
+
+    with ThreadPool(8) as pool:
+        results = pool.map(round_trip_transform, (pixel,) * 8)
+        for pixel2 in results:
+            assert_allclose(pixel, pixel2, atol=1e-7)
